import org.mockito.ArgumentMatchers.anyString
import org.mockito.ArgumentMatchersSugar
import org.mockito.scalatest.IdiomaticMockito
import org.scalatest.funsuite.AnyFunSuite

import scala.concurrent.duration.Duration
import scala.concurrent.{Await, Future}
import org.mockito.Mockito.doCallRealMethod

class SecurityPolicyTest extends AnyFunSuite with IdiomaticMockito with ArgumentMatchersSugar {

  test("Admin operation should not be called with user role") {
    implicit val securityPolicy: SecurityPolicy = mock[SecurityPolicy]
    val dataManager = new DataManager()

    { (role: UserRole, operation: Function0[Future[Int]], userRole: UserRole) =>
      throw new Exception("Access denied")
    } willBe answered by securityPolicy.withRole(eqTo(UserRole.ADMIN))(any[Future[Int]])(
      any[UserRole]
    )
    intercept[Exception] {
      implicit val userRole: UserRole = UserRole.USER
      Await.result(dataManager.doAdminStuff("test"), Duration.Inf)
    }
    assert(dataManager.adminOperationCount == 0)
  }
  test("using spy, Admin operation should not be called with user") {
    implicit val securityPolicy: SecurityPolicy = mock[SecurityPolicy]
    val dataManager = spy(new DataManager())

    { (role: UserRole, operation: Function0[Future[Int]], userRole: UserRole) =>
      throw new Exception("Access denied")
    } willBe answered by securityPolicy.withRole(eqTo(UserRole.ADMIN))(any[Future[Int]])(
      any[UserRole]
    )
    doCallRealMethod().when(dataManager).doAdminStuff(anyString)(any[SecurityPolicy], any[UserRole])
    doCallRealMethod().when(dataManager).adminOperationCount

    intercept[Exception] {
      implicit val userRole: UserRole = UserRole.USER
      Await.result(dataManager.doAdminStuff("test"), Duration.Inf)
    }
    assert(dataManager.adminOperationCount == 0)
  }
}
